@@ -23,6 +23,7 @@
 
 #include "windef.h"
 #include "winbase.h"
+#include "winternl.h"
 #include "objbase.h"
 #include "taskschd.h"
 #include "mstask.h"
@@ -57,11 +58,11 @@ typedef struct
     LONG ref;
     ITaskDefinition *task;
     IExecAction *action;
+    UUID uuid;
     LPWSTR task_name;
-    UINT flags;
     HRESULT status;
     WORD idle_minutes, deadline_minutes;
-    DWORD priority, maxRunTime;
+    DWORD flags, priority, maxRunTime, exit_code;
     LPWSTR accountName;
     DWORD trigger_count;
     TASK_TRIGGER *trigger;
@@ -144,15 +145,176 @@ static ULONG WINAPI MSTASK_ITask_Release(
     return ref;
 }
 
+HRESULT task_set_trigger(ITask *task, WORD idx, const TASK_TRIGGER *src)
+{
+    TaskImpl *This = impl_from_ITask(task);
+    TIME_FIELDS field_time;
+    LARGE_INTEGER sys_time;
+    TASK_TRIGGER dst;
+
+    TRACE("(%p, %u, %p)\n", task, idx, src);
+
+    if (idx >= This->trigger_count)
+        return E_FAIL;
+
+    /* Verify valid structure size */
+    if (src->cbTriggerSize != sizeof(*src))
+        return E_INVALIDARG;
+    dst.cbTriggerSize = src->cbTriggerSize;
+
+    /* Reserved field must be zero */
+    dst.Reserved1 = 0;
+
+    /* Verify and set valid start date and time */
+    memset(&field_time, 0, sizeof(field_time));
+    field_time.Year = src->wBeginYear;
+    field_time.Month = src->wBeginMonth;
+    field_time.Day = src->wBeginDay;
+    field_time.Hour = src->wStartHour;
+    field_time.Minute = src->wStartMinute;
+    if (!RtlTimeFieldsToTime(&field_time, &sys_time))
+        return E_INVALIDARG;
+    dst.wBeginYear = src->wBeginYear;
+    dst.wBeginMonth = src->wBeginMonth;
+    dst.wBeginDay = src->wBeginDay;
+    dst.wStartHour = src->wStartHour;
+    dst.wStartMinute = src->wStartMinute;
+
+    /* Verify valid end date if TASK_TRIGGER_FLAG_HAS_END_DATE flag is set */
+    if (src->rgFlags & TASK_TRIGGER_FLAG_HAS_END_DATE)
+    {
+        memset(&field_time, 0, sizeof(field_time));
+        field_time.Year = src->wEndYear;
+        field_time.Month = src->wEndMonth;
+        field_time.Day = src->wEndDay;
+        if (!RtlTimeFieldsToTime(&field_time, &sys_time))
+            return E_INVALIDARG;
+    }
+
+    /* Set valid end date independent of TASK_TRIGGER_FLAG_HAS_END_DATE flag */
+    dst.wEndYear = src->wEndYear;
+    dst.wEndMonth = src->wEndMonth;
+    dst.wEndDay = src->wEndDay;
+
+    /* Verify duration and interval pair */
+    if (src->MinutesDuration <= src->MinutesInterval && src->MinutesInterval > 0)
+        return E_INVALIDARG;
+    dst.MinutesDuration = src->MinutesDuration;
+    dst.MinutesInterval = src->MinutesInterval;
+
+    /* Copy over flags */
+    dst.rgFlags = src->rgFlags;
+
+    /* Set TriggerType dependent fields of Type union */
+    dst.TriggerType = src->TriggerType;
+    switch (src->TriggerType)
+    {
+        case TASK_TIME_TRIGGER_DAILY:
+            dst.Type.Daily.DaysInterval = src->Type.Daily.DaysInterval;
+            break;
+        case TASK_TIME_TRIGGER_WEEKLY:
+            dst.Type.Weekly.WeeksInterval = src->Type.Weekly.WeeksInterval;
+            dst.Type.Weekly.rgfDaysOfTheWeek = src->Type.Weekly.rgfDaysOfTheWeek;
+            break;
+        case TASK_TIME_TRIGGER_MONTHLYDATE:
+            dst.Type.MonthlyDate.rgfDays = src->Type.MonthlyDate.rgfDays;
+            dst.Type.MonthlyDate.rgfMonths = src->Type.MonthlyDate.rgfMonths;
+            break;
+        case TASK_TIME_TRIGGER_MONTHLYDOW:
+            dst.Type.MonthlyDOW.wWhichWeek = src->Type.MonthlyDOW.wWhichWeek;
+            dst.Type.MonthlyDOW.rgfDaysOfTheWeek = src->Type.MonthlyDOW.rgfDaysOfTheWeek;
+            dst.Type.MonthlyDOW.rgfMonths = src->Type.MonthlyDOW.rgfMonths;
+            break;
+        case TASK_TIME_TRIGGER_ONCE:
+        case TASK_EVENT_TRIGGER_ON_IDLE:
+        case TASK_EVENT_TRIGGER_AT_SYSTEMSTART:
+        case TASK_EVENT_TRIGGER_AT_LOGON:
+        default:
+            dst.Type = src->Type;
+            break;
+    }
+
+    /* Reserved field must be zero */
+    dst.Reserved2 = 0;
+
+    /* wRandomMinutesInterval not currently used and is initialized to zero */
+    dst.wRandomMinutesInterval = 0;
+
+    This->trigger[idx] = dst;
+
+    return S_OK;
+}
+
+HRESULT task_get_trigger(ITask *task, WORD idx, TASK_TRIGGER *dst)
+{
+    TaskImpl *This = impl_from_ITask(task);
+    TASK_TRIGGER *src;
+
+    TRACE("(%p, %u, %p)\n", task, idx, dst);
+
+    if (idx >= This->trigger_count)
+        return SCHED_E_TRIGGER_NOT_FOUND;
+
+    src = &This->trigger[idx];
+
+    /* Native implementation doesn't verify equivalent cbTriggerSize fields */
+
+    /* Copy relevant fields of the structure */
+    dst->cbTriggerSize = src->cbTriggerSize;
+    dst->Reserved1 = 0;
+    dst->wBeginYear = src->wBeginYear;
+    dst->wBeginMonth = src->wBeginMonth;
+    dst->wBeginDay = src->wBeginDay;
+    dst->wEndYear = src->wEndYear;
+    dst->wEndMonth = src->wEndMonth;
+    dst->wEndDay = src->wEndDay;
+    dst->wStartHour = src->wStartHour;
+    dst->wStartMinute = src->wStartMinute;
+    dst->MinutesDuration = src->MinutesDuration;
+    dst->MinutesInterval = src->MinutesInterval;
+    dst->rgFlags = src->rgFlags;
+    dst->TriggerType = src->TriggerType;
+    switch (src->TriggerType)
+    {
+        case TASK_TIME_TRIGGER_DAILY:
+            dst->Type.Daily.DaysInterval = src->Type.Daily.DaysInterval;
+            break;
+        case TASK_TIME_TRIGGER_WEEKLY:
+            dst->Type.Weekly.WeeksInterval = src->Type.Weekly.WeeksInterval;
+            dst->Type.Weekly.rgfDaysOfTheWeek = src->Type.Weekly.rgfDaysOfTheWeek;
+            break;
+        case TASK_TIME_TRIGGER_MONTHLYDATE:
+            dst->Type.MonthlyDate.rgfDays = src->Type.MonthlyDate.rgfDays;
+            dst->Type.MonthlyDate.rgfMonths = src->Type.MonthlyDate.rgfMonths;
+            break;
+        case TASK_TIME_TRIGGER_MONTHLYDOW:
+            dst->Type.MonthlyDOW.wWhichWeek = src->Type.MonthlyDOW.wWhichWeek;
+            dst->Type.MonthlyDOW.rgfDaysOfTheWeek = src->Type.MonthlyDOW.rgfDaysOfTheWeek;
+            dst->Type.MonthlyDOW.rgfMonths = src->Type.MonthlyDOW.rgfMonths;
+            break;
+        case TASK_TIME_TRIGGER_ONCE:
+        case TASK_EVENT_TRIGGER_ON_IDLE:
+        case TASK_EVENT_TRIGGER_AT_SYSTEMSTART:
+        case TASK_EVENT_TRIGGER_AT_LOGON:
+        default:
+            break;
+    }
+    dst->Reserved2 = 0;
+    dst->wRandomMinutesInterval = 0;
+
+    return S_OK;
+}
+
 static HRESULT WINAPI MSTASK_ITask_CreateTrigger(ITask *iface, WORD *idx, ITaskTrigger **task_trigger)
 {
     TaskImpl *This = impl_from_ITask(iface);
     TASK_TRIGGER *new_trigger;
+    SYSTEMTIME time;
     HRESULT hr;
 
     TRACE("(%p, %p, %p)\n", iface, idx, task_trigger);
 
-    hr = TaskTriggerConstructor((void **)task_trigger);
+    hr = TaskTriggerConstructor(iface, This->trigger_count, task_trigger);
     if (hr != S_OK) return hr;
 
     if (This->trigger)
@@ -167,9 +329,22 @@ static HRESULT WINAPI MSTASK_ITask_CreateTrigger(ITask *iface, WORD *idx, ITaskT
 
     This->trigger = new_trigger;
 
-    hr = ITaskTrigger_GetTrigger(*task_trigger, &This->trigger[This->trigger_count]);
-    if (hr == S_OK)
-        *idx = This->trigger_count++;
+    new_trigger = &This->trigger[This->trigger_count];
+
+    /* Most fields default to zero. Initialize other fields to default values. */
+    memset(new_trigger, 0, sizeof(*new_trigger));
+    GetLocalTime(&time);
+    new_trigger->cbTriggerSize = sizeof(*new_trigger);
+    new_trigger->wBeginYear = time.wYear;
+    new_trigger->wBeginMonth = time.wMonth;
+    new_trigger->wBeginDay = time.wDay;
+    new_trigger->wStartHour = time.wHour;
+    new_trigger->wStartMinute = time.wMinute;
+    new_trigger->rgFlags = TASK_TRIGGER_FLAG_DISABLED;
+    new_trigger->TriggerType = TASK_TIME_TRIGGER_DAILY,
+    new_trigger->Type.Daily.DaysInterval = 1;
+
+    *idx = This->trigger_count++;
 
     return hr;
 }
@@ -204,21 +379,13 @@ static HRESULT WINAPI MSTASK_ITask_GetTriggerCount(ITask *iface, WORD *count)
 static HRESULT WINAPI MSTASK_ITask_GetTrigger(ITask *iface, WORD idx, ITaskTrigger **trigger)
 {
     TaskImpl *This = impl_from_ITask(iface);
-    HRESULT hr;
 
     TRACE("(%p, %u, %p)\n", iface, idx, trigger);
 
     if (idx >= This->trigger_count)
         return SCHED_E_TRIGGER_NOT_FOUND;
 
-    hr = TaskTriggerConstructor((void **)trigger);
-    if (hr != S_OK) return hr;
-
-    hr = ITaskTrigger_SetTrigger(*trigger, &This->trigger[idx]);
-    if (hr != S_OK)
-        ITaskTrigger_Release(*trigger);
-
-    return hr;
+    return TaskTriggerConstructor(iface, idx, trigger);
 }
 
 static HRESULT WINAPI MSTASK_ITask_GetTriggerString(
@@ -242,12 +409,192 @@ static HRESULT WINAPI MSTASK_ITask_GetRunTimes(
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI MSTASK_ITask_GetNextRunTime(ITask *iface, SYSTEMTIME *st)
+static void get_begin_time(const TASK_TRIGGER *trigger, FILETIME *ft)
 {
-    FIXME("(%p, %p): stub\n", iface, st);
+    SYSTEMTIME st;
 
-    memset(st, 0, sizeof(*st));
-    return SCHED_S_TASK_NO_VALID_TRIGGERS;
+    st.wYear = trigger->wBeginYear;
+    st.wMonth = trigger->wBeginMonth;
+    st.wDay = trigger->wBeginDay;
+    st.wDayOfWeek = 0;
+    st.wHour = 0;
+    st.wMinute = 0;
+    st.wSecond = 0;
+    st.wMilliseconds = 0;
+    SystemTimeToFileTime(&st, ft);
+}
+
+static void get_end_time(const TASK_TRIGGER *trigger, FILETIME *ft)
+{
+    SYSTEMTIME st;
+
+    if (!(trigger->rgFlags & TASK_TRIGGER_FLAG_HAS_END_DATE))
+    {
+        ft->dwHighDateTime = ~0u;
+        ft->dwLowDateTime = ~0u;
+        return;
+    }
+
+    st.wYear = trigger->wEndYear;
+    st.wMonth = trigger->wEndMonth;
+    st.wDay = trigger->wEndDay;
+    st.wDayOfWeek = 0;
+    st.wHour = 0;
+    st.wMinute = 0;
+    st.wSecond = 0;
+    st.wMilliseconds = 0;
+    SystemTimeToFileTime(&st, ft);
+}
+
+static void filetime_add_ms(FILETIME *ft, ULONGLONG ms)
+{
+    union u_ftll
+    {
+        FILETIME ft;
+        ULONGLONG ll;
+    } *ftll = (union u_ftll *)ft;
+
+    ftll->ll += ms * (ULONGLONG)10000;
+}
+
+static void filetime_add_hours(FILETIME *ft, ULONG hours)
+{
+    filetime_add_ms(ft, (ULONGLONG)hours * 60 * 60 * 1000);
+}
+
+static void filetime_add_days(FILETIME *ft, ULONG days)
+{
+    filetime_add_hours(ft, (ULONGLONG)days * 24);
+}
+
+static void filetime_add_weeks(FILETIME *ft, ULONG weeks)
+{
+    filetime_add_days(ft, (ULONGLONG)weeks * 7);
+}
+
+static HRESULT WINAPI MSTASK_ITask_GetNextRunTime(ITask *iface, SYSTEMTIME *rt)
+{
+    TaskImpl *This = impl_from_ITask(iface);
+    HRESULT hr = SCHED_S_TASK_NO_VALID_TRIGGERS;
+    SYSTEMTIME st, current_st;
+    FILETIME current_ft, begin_ft, end_ft, best_ft;
+    BOOL have_best_time = FALSE;
+    DWORD i;
+
+    TRACE("(%p, %p)\n", iface, rt);
+
+    if (This->flags & TASK_FLAG_DISABLED)
+    {
+        memset(rt, 0, sizeof(*rt));
+        return SCHED_S_TASK_DISABLED;
+    }
+
+    GetLocalTime(&current_st);
+
+    for (i = 0; i < This->trigger_count; i++)
+    {
+        if (!(This->trigger[i].rgFlags & TASK_TRIGGER_FLAG_DISABLED))
+        {
+            get_begin_time(&This->trigger[i], &begin_ft);
+            get_end_time(&This->trigger[i], &end_ft);
+
+            switch (This->trigger[i].TriggerType)
+            {
+            case TASK_EVENT_TRIGGER_ON_IDLE:
+            case TASK_EVENT_TRIGGER_AT_SYSTEMSTART:
+            case TASK_EVENT_TRIGGER_AT_LOGON:
+                hr = SCHED_S_EVENT_TRIGGER;
+                break;
+
+            case TASK_TIME_TRIGGER_ONCE:
+                st = current_st;
+                st.wHour = This->trigger[i].wStartHour;
+                st.wMinute = This->trigger[i].wStartMinute;
+                st.wSecond = 0;
+                st.wMilliseconds = 0;
+                SystemTimeToFileTime(&st, &current_ft);
+                if (CompareFileTime(&begin_ft, &current_ft) <= 0 && CompareFileTime(&current_ft, &end_ft) < 0)
+                {
+                    if (!have_best_time || CompareFileTime(&current_ft, &best_ft) < 0)
+                    {
+                        best_ft = current_ft;
+                        have_best_time = TRUE;
+                    }
+                }
+                break;
+
+            case TASK_TIME_TRIGGER_DAILY:
+                st = current_st;
+                st.wHour = This->trigger[i].wStartHour;
+                st.wMinute = This->trigger[i].wStartMinute;
+                st.wSecond = 0;
+                st.wMilliseconds = 0;
+                SystemTimeToFileTime(&st, &current_ft);
+                while (CompareFileTime(&current_ft, &end_ft) < 0)
+                {
+                    if (CompareFileTime(&current_ft, &begin_ft) >= 0)
+                    {
+                        if (!have_best_time || CompareFileTime(&current_ft, &best_ft) < 0)
+                        {
+                            best_ft = current_ft;
+                            have_best_time = TRUE;
+                        }
+                        break;
+                    }
+
+                    filetime_add_days(&current_ft, This->trigger[i].Type.Daily.DaysInterval);
+                }
+                break;
+
+            case TASK_TIME_TRIGGER_WEEKLY:
+                if (!This->trigger[i].Type.Weekly.rgfDaysOfTheWeek)
+                    break; /* avoid infinite loop */
+
+                st = current_st;
+                st.wHour = This->trigger[i].wStartHour;
+                st.wMinute = This->trigger[i].wStartMinute;
+                st.wSecond = 0;
+                st.wMilliseconds = 0;
+                SystemTimeToFileTime(&st, &current_ft);
+                while (CompareFileTime(&current_ft, &end_ft) < 0)
+                {
+                    FileTimeToSystemTime(&current_ft, &st);
+
+                    if (CompareFileTime(&current_ft, &begin_ft) >= 0)
+                    {
+                        if (This->trigger[i].Type.Weekly.rgfDaysOfTheWeek & (1 << st.wDayOfWeek))
+                        {
+                            if (!have_best_time || CompareFileTime(&current_ft, &best_ft) < 0)
+                            {
+                                best_ft = current_ft;
+                                have_best_time = TRUE;
+                            }
+                            break;
+                        }
+                    }
+
+                    if (st.wDayOfWeek == 0 && This->trigger[i].Type.Weekly.WeeksInterval > 1) /* Sunday, goto next week */
+                        filetime_add_weeks(&current_ft, This->trigger[i].Type.Weekly.WeeksInterval - 1);
+                    else /* check next weekday */
+                        filetime_add_days(&current_ft, 1);
+                }
+                break;
+
+            default:
+                FIXME("trigger type %u is not handled\n", This->trigger[i].TriggerType);
+                break;
+            }
+        }
+    }
+
+    if (have_best_time)
+    {
+        FileTimeToSystemTime(&best_ft, rt);
+        return S_OK;
+    }
+
+    memset(rt, 0, sizeof(*rt));
+    return hr;
 }
 
 static HRESULT WINAPI MSTASK_ITask_SetIdleWait(
@@ -325,10 +672,12 @@ static HRESULT WINAPI MSTASK_ITask_GetStatus(ITask *iface, HRESULT *status)
 
 static HRESULT WINAPI MSTASK_ITask_GetExitCode(ITask *iface, DWORD *exit_code)
 {
-    FIXME("(%p, %p): stub\n", iface, exit_code);
+    TaskImpl *This = impl_from_ITask(iface);
 
-    *exit_code = 0;
-    return SCHED_S_TASK_HAS_NOT_RUN;
+    TRACE("(%p, %p)\n", iface, exit_code);
+
+    *exit_code = This->exit_code;
+    return SCHED_S_TASK_HAS_NOT_RUN; /* FIXME */
 }
 
 static HRESULT WINAPI MSTASK_ITask_SetComment(ITask *iface, LPCWSTR comment)
@@ -491,11 +840,14 @@ static HRESULT WINAPI MSTASK_ITask_GetErrorRetryInterval(ITask *iface, WORD *int
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI MSTASK_ITask_SetFlags(
-        ITask* iface,
-        DWORD dwFlags)
+static HRESULT WINAPI MSTASK_ITask_SetFlags(ITask *iface, DWORD flags)
 {
-    FIXME("(%p, 0x%08x): stub\n", iface, dwFlags);
+    TaskImpl *This = impl_from_ITask(iface);
+
+    TRACE("(%p, 0x%08x)\n", iface, flags);
+    This->flags &= 0xffff8000;
+    This->flags |= flags & 0x7fff;
+    This->is_dirty = TRUE;
     return S_OK;
 }
 
@@ -890,6 +1242,8 @@ static HRESULT load_job_data(TaskImpl *This, BYTE *data, DWORD size)
     if (fixed->file_version != 0x0001)
         return SCHED_E_INVALID_TASK;
 
+    This->uuid = fixed->uuid;
+
     TRACE("name_size_offset %04x\n", fixed->name_size_offset);
     TRACE("trigger_offset %04x\n", fixed->trigger_offset);
     TRACE("error_retry_count %u\n", fixed->error_retry_count);
@@ -903,6 +1257,7 @@ static HRESULT load_job_data(TaskImpl *This, BYTE *data, DWORD size)
     TRACE("maximum_runtime %u\n", fixed->maximum_runtime);
     This->maxRunTime = fixed->maximum_runtime;
     TRACE("exit_code %#x\n", fixed->exit_code);
+    This->exit_code = fixed->exit_code;
     TRACE("status %08x\n", fixed->status);
     This->status = fixed->status;
     TRACE("flags %08x\n", fixed->flags);
@@ -1141,46 +1496,41 @@ static BOOL write_user_data(HANDLE hfile, BYTE *data, WORD data_size)
     return WriteFile(hfile, data, data_size, &size, NULL);
 }
 
-static HRESULT write_triggers(ITask *task, HANDLE hfile)
+static HRESULT write_triggers(TaskImpl *This, HANDLE hfile)
 {
     WORD count, i, idx = 0xffff;
     DWORD size;
-    HRESULT hr;
+    HRESULT hr = S_OK;
     ITaskTrigger *trigger;
 
-    hr = ITask_GetTriggerCount(task, &count);
-    if (hr != S_OK) return hr;
+    count = This->trigger_count;
 
     /* Windows saves a .job with at least 1 trigger */
     if (!count)
     {
-        hr = ITask_CreateTrigger(task, &idx, &trigger);
+        hr = ITask_CreateTrigger(&This->ITask_iface, &idx, &trigger);
         if (hr != S_OK) return hr;
         ITaskTrigger_Release(trigger);
 
         count = 1;
     }
 
-    if (!WriteFile(hfile, &count, sizeof(count), &size, NULL))
-        return HRESULT_FROM_WIN32(GetLastError());
-
-    for (i = 0; i < count; i++)
+    if (WriteFile(hfile, &count, sizeof(count), &size, NULL))
     {
-        TASK_TRIGGER task_trigger;
-
-        hr = ITask_GetTrigger(task, i, &trigger);
-        if (hr != S_OK) return hr;
-
-        hr = ITaskTrigger_GetTrigger(trigger, &task_trigger);
-        ITaskTrigger_Release(trigger);
-        if (hr != S_OK) break;
-
-        if (!WriteFile(hfile, &task_trigger, sizeof(task_trigger), &size, NULL))
-            return HRESULT_FROM_WIN32(GetLastError());
+        for (i = 0; i < count; i++)
+        {
+            if (!WriteFile(hfile, &This->trigger[i], sizeof(This->trigger[0]), &size, NULL))
+            {
+                hr = HRESULT_FROM_WIN32(GetLastError());
+                break;
+            }
+        }
     }
+    else
+        hr = HRESULT_FROM_WIN32(GetLastError());
 
     if (idx != 0xffff)
-        ITask_DeleteTrigger(task, idx);
+        ITask_DeleteTrigger(&This->ITask_iface, idx);
 
     return hr;
 }
@@ -1236,7 +1586,6 @@ static HRESULT WINAPI MSTASK_IPersistFile_Save(IPersistFile *iface, LPCOLESTR ta
     ver = GetVersion();
     fixed.product_version = MAKEWORD(ver >> 8, ver);
     fixed.file_version = 0x0001;
-    CoCreateGuid(&fixed.uuid);
     fixed.name_size_offset = sizeof(fixed) + sizeof(USHORT); /* FIXDLEN_DATA + Instance Count */
     fixed.trigger_offset = sizeof(fixed) + sizeof(USHORT); /* FIXDLEN_DATA + Instance Count */
     fixed.trigger_offset += sizeof(USHORT); /* Application Name */
@@ -1263,7 +1612,7 @@ static HRESULT WINAPI MSTASK_IPersistFile_Save(IPersistFile *iface, LPCOLESTR ta
     fixed.idle_deadline = This->deadline_minutes;
     fixed.priority = This->priority;
     fixed.maximum_runtime = This->maxRunTime;
-    fixed.exit_code = 0;
+    fixed.exit_code = This->exit_code;
     if (This->status == SCHED_S_TASK_NOT_SCHEDULED && This->trigger_count)
         This->status = SCHED_S_TASK_HAS_NOT_RUN;
     fixed.status = This->status;
@@ -1280,6 +1629,11 @@ static HRESULT WINAPI MSTASK_IPersistFile_Save(IPersistFile *iface, LPCOLESTR ta
         Sleep(100);
     }
 
+    if (GetLastError() == ERROR_ALREADY_EXISTS)
+        fixed.uuid = This->uuid;
+    else
+        CoCreateGuid(&fixed.uuid);
+
     if (!WriteFile(hfile, &fixed, sizeof(fixed), &size, NULL))
     {
         hr = HRESULT_FROM_WIN32(GetLastError());
@@ -1338,7 +1692,7 @@ static HRESULT WINAPI MSTASK_IPersistFile_Save(IPersistFile *iface, LPCOLESTR ta
     }
 
     /* Triggers */
-    hr = write_triggers(task, hfile);
+    hr = write_triggers(This, hfile);
     if (hr != S_OK)
         goto failed;
 
@@ -1491,6 +1845,7 @@ HRESULT TaskConstructor(ITaskService *service, const WCHAR *name, ITask **task)
     This->task_name = heap_strdupW(task_name);
     This->flags = 0;
     This->status = SCHED_S_TASK_NOT_SCHEDULED;
+    This->exit_code = 0;
     This->idle_minutes = 10;
     This->deadline_minutes = 60;
     This->priority = NORMAL_PRIORITY_CLASS;
@@ -1500,6 +1855,8 @@ HRESULT TaskConstructor(ITaskService *service, const WCHAR *name, ITask **task)
     This->is_dirty = FALSE;
     This->instance_count = 0;
 
+    CoCreateGuid(&This->uuid);
+
     /* Default time is 3 days = 259200000 ms */
     This->maxRunTime = 259200000;
 
