@@ -23,6 +23,8 @@
 #include "wine/unicode.h"
 
 void schedsvc_auto_start(void) DECLSPEC_HIDDEN;
+void add_job(const WCHAR *name) DECLSPEC_HIDDEN;
+void remove_job(const WCHAR *name) DECLSPEC_HIDDEN;
 
 static inline WCHAR *heap_strdupW(const WCHAR *src)
 {
